# frozen_string_literal: true

require "async"
Console.logger.off!

module Is
  # [public] Makes Ruby objects async-aware.
  #
  module Async
    require_relative "async/timeout"
    require_relative "async/version"

    # [public] Call asynchronous behavior in a proper async context.
    #
    def async
      ::Async::Reactor.run do |task|
        yield task
      end
    end

    # [public] Call asynchronous behavior synchronously in a proper async context.
    #
    def await
      if (task = ::Async::Task.current?)
        yield task
      else
        ::Async::Reactor.run { |task|
          yield task
        }.wait
      end
    end

    # [public] Sleeps for `seconds` in a proper async context.
    #
    def sleep(seconds)
      await do |task|
        task.sleep(seconds)
      end
    end

    # [public] Call asynchonous behavior in a proper async context, wrapped in a timeout.
    #
    # Raises `Is::Async::Timeout` if execution exceeds `seconds`.
    #
    def timeout(seconds, &block)
      await do |task|
        if seconds && seconds > 0
          task.with_timeout(seconds, Timeout) do
            yield task
          end
        else
          yield task
        end
      end
    end
  end
end
